#include "CKlearAppStreamReader.h"

CKlearAppStreamReader::CKlearAppStreamReader(std::vector<CKlearAppEITData *> *storage, CKlearAppConfig *config)
{
  this->decoder = new CKlearAppDataDecoder(storage, config);
  this->running = true;
  this->setUpDemux();
}

CKlearAppStreamReader::~CKlearAppStreamReader()
{
   this->tearDownDemux();
   delete decoder;
}

void CKlearAppStreamReader::readContent()
{
   this->readSectorLoop();
}

void CKlearAppStreamReader::run()
{
  std::cout << "Reading in Content" << std::endl;
  this->readContent();
}

void CKlearAppStreamReader::setRunning(bool v)
{
   this->running = v;
}

bool CKlearAppStreamReader::setUpDemux()
{
  if((this->DemuxFD = open(DEMUX_DEVICE,O_RDWR)) < 0){
      std::cerr << "EIT EPG: Could not open demux device for reading!" << std::endl;
      return(false);
  }

  if( ioctl(this->DemuxFD, DMX_SET_BUFFER_SIZE, SECT_BUF_SIZE) < 0 ) {
      std::cerr << "DMX_SET_BUFFER_SIZE failed" << std::endl;
      this->tearDownDemux();
      return(false);
  }

   struct dmx_sct_filter_params flt;
   memset( &flt, 0, sizeof(struct dmx_sct_filter_params) );
   flt.pid = EIT_PID;
   flt.filter.filter[0] = 0;
   flt.filter.mask[0] = 0;
   flt.timeout = 0;
   flt.flags = DMX_IMMEDIATE_START; // |= DMX_CHECK_CRC;

   if (ioctl (this->DemuxFD, DMX_SET_FILTER, &flt) < 0) {
      std::cerr << "DMX_SET_FILTER failed" << std::endl;
      this->tearDownDemux();
      return(false);
   }

   return true;
}

void CKlearAppStreamReader::tearDownDemux()
{
  ioctl (this->DemuxFD, DMX_STOP, 0);
  close(this->DemuxFD);
}


long CKlearAppStreamReader::readBuffer()
{
    int    n;
    int    sect_len;

    n = read(this->DemuxFD, this->buf,3);             // read section header
    if (n <= 0) return n;           // error or strange, abort

    sect_len = ((buf[1] & 0x0F) << 8) + buf[2]; // get section size
    if (sect_len > (sizeof(this->buf)-3)) return -1;   // something odd?

    n = read(this->DemuxFD,this->buf+3,sect_len);        // read 1 section
    if (n >=0)
        n += 3;              // we already read header bytes

    return n;
}

void CKlearAppStreamReader::readSectorLoop()
{
  while( running == true )  // to stop reading, set running to false
  {
    long n = readBuffer(); // modifies this->buf with demux data

    if (n < 0) { // -- error or eof?
      if (errno == ETIMEDOUT) break;        // Timout, abort
      continue;
    }

    if (n == 0) continue;  // dmxmode = no eof!

    // give parsed buffer data to decoder
    this->decoder->decodeBuffer( this->buf, n );
  }
}
