#include <iostream>
#include <qlistview.h>
#include "CKlearControllerEPGMain.h"

CKlearControllerEPGMain::CKlearControllerEPGMain(std::vector<CKlearAppEITData *> *storage, QString chan,QWidget* parent, const char* name, WFlags fl) : CKlearUIEPGMain(parent,name,fl)
{
  this->EITDataVector = storage; // map storage vector to local member (static is still thread local, so no direct access possible)

  this->currentChannel = chan;

  // set channelname to EPG label header
  QString epgl(EPG_Label->text()+"<h2>for "+this->currentChannel+"</h2>");
  EPG_Label->setText(epgl);

   // set today to date selector list
   EPGDateBox->insertItem( QDateTime::currentDateTime().date().toString() );
   EPGDateBox->setCurrentItem(0);

  this->slotRefreshView(); // show all already collected data on GUI startup
}

CKlearControllerEPGMain::~CKlearControllerEPGMain()
{
}

void CKlearControllerEPGMain::slotRebuildDatabase()
{
  std::cout << "Cleaning up data storage vector" << std::endl;

  CKlearAppDataLayer *DataLayer = new CKlearAppDataLayer( this->EITDataVector );
  DataLayer->cleanUpDatabase();
  delete DataLayer;

  EPGLongDescBox->clear(); // clean up description box
  this->slotRefreshView(); // clean up GUI and refresh view
}

void CKlearControllerEPGMain::slotRefreshView()
{
  std::cout << "Refreshing GUI with collected vector data" << std::endl;
  EPGProgramDataBox->clear();

  std::cout << "Vectorsize: " << this->EITDataVector->size() << std::endl;

  std::cout << "Timeshift calculated: "
  << QDateTime::currentDateTime ( Qt::LocalTime ).toTime_t() -   QDateTime::currentDateTime ( Qt::UTC ).toTime_t()  << std::endl;
  int TIMESHIFT = QDateTime::currentDateTime ( Qt::LocalTime ).toTime_t() -   QDateTime::currentDateTime ( Qt::UTC ).toTime_t();

   for( std::vector<CKlearAppEITData *>::iterator it = this->EITDataVector->begin(); it != this->EITDataVector->end(); ++it )
   {
         this->ST.setTime_t( (*it)->StartTime+TIMESHIFT );
         if( ! isInDateList() ) // check currently set ST
               EPGDateBox->insertItem( this->ST.date().toString() ); // add to date selector list

         if( EPGDateBox->text(  EPGDateBox->currentItem() ) ==  this->ST.date().toString() ) // if data are from selected date
         {
             QListViewItem *item = new QListViewItem( EPGProgramDataBox, ST.toString("hh:mm"), (*it)->ShortDesc  );
             EPGProgramDataBox->insertItem( item );

             // pre-select if current running program
             if( ( (*it)->StartTime < QDateTime::currentDateTime(Qt::UTC).toTime_t() )
                    && ( (*it)->StartTime+(*it)->Duration > QDateTime::currentDateTime(Qt::UTC).toTime_t() ) )
             {
                  std::cout << "current item found, marking..." << std::endl;
                  this->EPGProgramDataBox->setCurrentItem(  item );
             }
         }
   }

}


bool CKlearControllerEPGMain::isInDateList()
{
    for( uint i = 0; i <= EPGDateBox->count(); i++)
   {
        if( this->ST.date().toString() == EPGDateBox->text(i) )
            return true;
   }

   return false;
}

void CKlearControllerEPGMain::slotShowItemDetails()
{
   QDateTime TempDateStart;  // temp. Date and Time object to identify selected object
   QDateTime TempDateEnd;  // temp. Date and Time object to claculate end time

   int TIMESHIFT = QDateTime::currentDateTime ( Qt::LocalTime ).toTime_t() -   QDateTime::currentDateTime ( Qt::UTC ).toTime_t();

   for( std::vector<CKlearAppEITData *>::iterator it = this->EITDataVector->begin(); it != this->EITDataVector->end(); ++it )
   {
       TempDateStart.setTime_t( (*it)->StartTime+TIMESHIFT );

       if(  ( TempDateStart.toString("hh:mm") == EPGProgramDataBox->currentItem()->text(0))  &&
            ( TempDateStart.date().toString() ==  EPGDateBox->currentText() ) ){

            // calculate end-time
            TempDateEnd.setTime_t( (*it)->StartTime+TIMESHIFT+(*it)->Duration );

            // object found, now add text
            QString LongDescription("");

            LongDescription = LongDescription + "<h3>" + (*it)->ShortDesc + "</h3>"; // program name

            //LongDescription = LongDescription + this->currentChannel + "<br>"; // channel name

            LongDescription = LongDescription + "<h4>" + TempDateStart.time().toString("hh:mm") + " - " + TempDateEnd.time().toString("hh:mm") + " o'clock "; // time
            LongDescription = LongDescription + "( " + QString().setNum( (*it)->Duration/60 )  + " min. )</h4>"; // duration

            LongDescription = LongDescription + "<i>Genre:</i> " + findTableID(ContentTable, (*it)->Genre) + "<br>"; // Genre
            LongDescription = LongDescription + "<i>Age:</i> " + findTableID(ParentalRatingTable, (*it)->Rating) + "<br>"; // Age
            LongDescription = LongDescription + "<i>Conditional access:</i> " + QString().setNum((*it)->CA) + "<br><p>"; // CA

            LongDescription = LongDescription+ (*it)->LongDesc;  // desc.

            EPGLongDescBox->setText( LongDescription );
       }
   }
}

void CKlearControllerEPGMain::slotActivateScheduledRecording()
{
   std::cout << "Starting scheduled recording for item" << std::endl;
}

void CKlearControllerEPGMain::slotCloseEPGwindow()
{
   this->close();
}
